@@ -1,7 +1,7 @@
 /*
  * lftp - file transfer program
  *
- * Copyright (c) 1996-2012 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2013 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,8 +17,6 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* $Id: Http.cc,v 1.242 2011/04/29 04:58:27 lav Exp $ */
-
 #include <config.h>
 #include "trio.h"
 #include <unistd.h>
@@ -38,6 +36,7 @@
 #include "HttpDir.h"
 #include "misc.h"
 #include "buffer_ssl.h"
+#include "buffer_zlib.h"
 
 #include "ascii_ctype.h"
 
@@ -187,11 +186,12 @@ void Http::ResetRequestData()
    sent_eot=false;
    keep_alive=false;
    keep_alive_max=-1;
-   array_send=array_ptr;
+   array_send=fileset_for_info?fileset_for_info->curr_index():0;
    chunked=false;
    chunk_size=-1;
    chunk_pos=0;
    chunked_trailer=false;
+   inflate=0;
    seen_ranges_bytes=false;
 }
 
@@ -406,22 +406,25 @@ bool Http::ModeSupported()
    abort(); // should not happen
 }
 
-void Http::DirFile(xstring& path,const char *ecwd,const char *efile)
+void Http::DirFile(xstring& path,const xstring& ecwd,const xstring& efile) const
 {
-   int base=path.length();
-   if(!strcmp(ecwd,"~") && !hftp)
-      ecwd="";
-   const char *sep=(last_char(ecwd)=='/'?"":"/");
-   if(efile[0]==0)
-      sep="";
-   const char *pre=(ecwd[0]=='/'?"":"/");
-   if(efile[0]=='/')
-      path.append(efile);
-   else if(efile[0]=='~')
-      path.vappend("/",efile,NULL);
-   else
-      path.vappend(pre,ecwd,sep,efile,NULL);
+   const int base=path.length();
 
+   if(efile[0]=='/') {
+      path.append(efile);
+   } else if(efile[0]=='~' || ecwd.length()==0 || (ecwd.eq("~") && !hftp)) {
+      path.append('/');
+      path.append(efile);
+   } else {
+      if(ecwd[0]!='/')
+	 path.append('/');
+      path.append(ecwd);
+      if(ecwd.last_char()!='/' && efile.length()>0)
+	 path.append('/');
+      path.append(efile);
+   }
+
+   // remove "/~" or "/~/"
    if(path[base+1]=='~' && path[base+2]==0)
       path.truncate(base+1);
    else if(path[base+1]=='~' && path[base+2]=='/')
@@ -681,6 +684,7 @@ void Http::SendRequest(const char *connection,const char *f)
    chunk_size=-1;
    chunk_pos=0;
    chunked_trailer=false;
+   inflate=0;
    no_ranges=false;
 
    conn->send_buf->SetPos(0);
@@ -695,11 +699,16 @@ void Http::SendArrayInfoRequest()
       if(m==-1)
 	 m=100;
    }
-   while(array_send-array_ptr<m && array_send<array_cnt)
+   while(array_send-fileset_for_info->curr_index()<m
+   && array_send<fileset_for_info->count())
    {
-      SendRequest(array_send==array_cnt-1 ? 0 : "keep-alive",
-	 array_for_info[array_send].file);
-      array_send++;
+      FileInfo *fi=(*fileset_for_info)[array_send++];
+      xstring *name=&fi->name;
+      if(fi->filetype==fi->DIRECTORY && name->last_char()!='/') {
+	 name=&xstring::get_tmp(*name);
+	 name->append('/');
+      }
+      SendRequest(array_send==fileset_for_info->count()-1 ? 0 : "keep-alive", *name);
    }
 }
 
@@ -738,15 +747,15 @@ void Http::HandleHeaderLine(const char *name,const char *value)
       if(bs<0) // try to workaround broken servers
 	 bs+=0x100000000LL;
       body_size=bs;
-      if(pos==0 && mode!=STORE && mode!=MAKE_DIR)
+      if(pos==0 && mode!=STORE && mode!=MAKE_DIR && !inflate)
 	 entity_size=body_size;
-      if(pos==0 && opt_size && H_20X(status_code))
+      if(pos==0 && opt_size && H_20X(status_code) && !inflate)
 	 *opt_size=body_size;
 
       if(mode==ARRAY_INFO && H_20X(status_code))
       {
-	 array_for_info[array_ptr].size=body_size;
-	 array_for_info[array_ptr].get_size=false;
+	 FileInfo *fi=fileset_for_info->curr();
+	 fi->SetSize(body_size);
 	 TrySuccess();
       }
       return;
@@ -783,8 +792,8 @@ void Http::HandleHeaderLine(const char *name,const char *value)
 
       if(mode==ARRAY_INFO && H_20X(status_code))
       {
-	 array_for_info[array_ptr].time=t;
-	 array_for_info[array_ptr].get_time=false;
+	 FileInfo *fi=fileset_for_info->curr();
+	 fi->SetDate(t,0);
 	 TrySuccess();
       }
       return;
@@ -831,9 +840,20 @@ void Http::HandleHeaderLine(const char *name,const char *value)
 	 chunk_pos=0;
 	 chunked_trailer=false;
       }
-      // handle gzip?
       return;
    }
+   if(!strcasecmp(name,"Content-Encoding"))
+   {
+      if(!strcasecmp(value,"gzip")) {
+	 // inflated size if unknown beforehand
+	 entity_size=NO_SIZE;
+	 if(opt_size)
+	    *opt_size=NO_SIZE;
+	 // start the inflation
+	 inflate=new DirectedBuffer(DirectedBuffer::GET);
+	 inflate->SetTranslator(new DataInflator());
+      }
+   }
    if(!strcasecmp(name,"Accept-Ranges"))
    {
       if(!strcasecmp(value,"none"))
@@ -1243,11 +1263,7 @@ int Http::Do()
 		  // we'll have to receive next header
 		  status.set(0);
 		  status_code=0;
-		  if(array_for_info[array_ptr].get_time)
-		     array_for_info[array_ptr].time=NO_DATE;
-		  if(array_for_info[array_ptr].get_size)
-		     array_for_info[array_ptr].size=NO_SIZE;
-		  if(++array_ptr>=array_cnt)
+		  if(!fileset_for_info->next())
 		  {
 		     state=DONE;
 		     return MOVED;
@@ -1575,6 +1591,7 @@ void  Http::ClassInit()
 
 void Http::SuspendInternal()
 {
+   super::SuspendInternal();
    if(conn)
       conn->SuspendInternal();
 }
@@ -1582,6 +1599,7 @@ void Http::ResumeInternal()
 {
    if(conn)
       conn->ResumeInternal();
+   super::ResumeInternal();
 }
 
 int Http::Read(void *buf,int size)
@@ -1606,6 +1624,18 @@ int Http::Read(void *buf,int size)
    }
    return res;
 }
+void Http::_Skip(int to_skip)
+{
+   if(inflate) {
+      inflate->Skip(to_skip);
+   } else {
+      conn->recv_buf->Skip(to_skip);
+      if(chunked)
+	 chunk_pos+=to_skip;
+      bytes_received+=to_skip;
+   }
+   real_pos+=to_skip;
+}
 int Http::_Read(void *buf,int size)
 {
    const char *buf1;
@@ -1633,7 +1663,8 @@ get_again:
    }
    if(!chunked)
    {
-      if(body_size>=0 && bytes_received>=body_size)
+      if(body_size>=0 && bytes_received>=body_size
+      && (!inflate || inflate->Size()==0))
       {
 	 LogNote(9,_("Received all"));
 	 return 0; // all received
@@ -1644,9 +1675,9 @@ get_again:
 	 return 0;
       }
    }
-   if(size1==0)
+   if(size1==0 && (!inflate || inflate->Size()==0))
       return DO_AGAIN;
-   if(chunked)
+   if(chunked && size1>0)
    {
       if(chunked_trailer && state==RECEIVING_HEADER)
 	 return DO_AGAIN;
@@ -1697,7 +1728,8 @@ get_again:
       if(size1>chunk_size-chunk_pos)
 	 size1=chunk_size-chunk_pos;
    }
-   else
+
+   if(!chunked)
    {
       // limit by body_size.
       if(body_size>=0 && size1+bytes_received>=body_size)
@@ -1707,8 +1739,23 @@ get_again:
    int bytes_allowed=0x10000000;
    if(rate_limit)
       bytes_allowed=rate_limit->BytesAllowedToGet();
-   if(size1>bytes_allowed)
-      size1=bytes_allowed;
+
+   if(inflate) {
+      // do the inflation, if there are not enough inflated data
+      if(size1>bytes_allowed)
+	 size1=bytes_allowed;
+      if(inflate->Size()<size && size1>0) {
+	 inflate->PutTranslated(buf1,size1);
+	 conn->recv_buf->Skip(size1);
+	 if(chunked)
+	    chunk_pos+=size1;
+	 bytes_received+=size1;
+      }
+      inflate->Get(&buf1,&size1);
+   } else {
+      if(size1>bytes_allowed)
+	 size1=bytes_allowed;
+   }
    if(size1==0)
       return DO_AGAIN;
    if(norest_manual && real_pos==0 && pos>0)
@@ -1718,21 +1765,13 @@ get_again:
       off_t to_skip=pos-real_pos;
       if(to_skip>size1)
 	 to_skip=size1;
-      conn->recv_buf->Skip(to_skip);
-      real_pos+=to_skip;
-      bytes_received+=to_skip;
-      if(chunked)
-	 chunk_pos+=to_skip;
+      _Skip(to_skip);
       goto get_again;
    }
    if(size>size1)
       size=size1;
    memcpy(buf,buf1,size);
-   conn->recv_buf->Skip(size);
-   if(chunked)
-      chunk_pos+=size;
-   real_pos+=size;
-   bytes_received+=size;
+   _Skip(size);
    return size;
 }
 
