@@ -1,7 +1,7 @@
 /*
- * lftp and utils
+ * lftp - file transfer program
  *
- * Copyright (c) 1996-2010 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2012 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -14,12 +14,9 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* $Id: MirrorJob.cc,v 1.176 2011/03/25 14:35:42 lav Exp $ */
-
 #include <config.h>
 
 #include <sys/types.h>
@@ -46,7 +43,7 @@
    Log::global->Format(11,"mirror(%p) enters state %s\n", this, #s); } while(0)
 #define waiting_num waiting.count()
 
-void  MirrorJob::PrintStatus(int v,const char *tab)
+xstring& MirrorJob::FormatStatus(xstring& s,int v,const char *tab)
 {
    if(Done())
       goto final;
@@ -64,60 +61,61 @@ void  MirrorJob::PrintStatus(int v,const char *tab)
       break;
 
    case(MAKE_TARGET_DIR):
-      printf("\tmkdir `%s' [%s]\n",target_dir.get(),target_session->CurrentStatus());
+      s.appendf("\tmkdir `%s' [%s]\n",target_dir.get(),target_session->CurrentStatus());
       break;
 
    case(CHANGING_DIR_SOURCE):
    case(CHANGING_DIR_TARGET):
       if(target_session->IsOpen())
-	 printf("\tcd `%s' [%s]\n",target_dir.get(),target_session->CurrentStatus());
+	 s.appendf("\tcd `%s' [%s]\n",target_dir.get(),target_session->CurrentStatus());
       if(source_session->IsOpen())
-	 printf("\tcd `%s' [%s]\n",source_dir.get(),source_session->CurrentStatus());
+	 s.appendf("\tcd `%s' [%s]\n",source_dir.get(),source_session->CurrentStatus());
       break;
 
    case(GETTING_LIST_INFO):
       if(target_list_info)
       {
 	 if(target_relative_dir)
-	    printf("\t%s: %s\n",target_relative_dir.get(),target_list_info->Status());
+	    s.appendf("\t%s: %s\n",target_relative_dir.get(),target_list_info->Status());
 	 else
-	    printf("\t%s\n",target_list_info->Status());
+	    s.appendf("\t%s\n",target_list_info->Status());
       }
       if(source_list_info)
       {
 	 if(source_relative_dir)
-	    printf("\t%s: %s\n",source_relative_dir.get(),source_list_info->Status());
+	    s.appendf("\t%s: %s\n",source_relative_dir.get(),source_list_info->Status());
 	 else
-	    printf("\t%s\n",source_list_info->Status());
+	    s.appendf("\t%s\n",source_list_info->Status());
       }
       break;
    }
-   return;
+   return s;
 
 final:
    if(stats.dirs>0)
-      printf(plural("%sTotal: %d director$y|ies$, %d file$|s$, %d symlink$|s$\n",
+      s.appendf(plural("%sTotal: %d director$y|ies$, %d file$|s$, %d symlink$|s$\n",
 		     stats.dirs,stats.tot_files,stats.tot_symlinks),
 	 tab,stats.dirs,stats.tot_files,stats.tot_symlinks);
    if(stats.new_files || stats.new_symlinks)
-      printf(plural("%sNew: %d file$|s$, %d symlink$|s$\n",
+      s.appendf(plural("%sNew: %d file$|s$, %d symlink$|s$\n",
 		     stats.new_files,stats.new_symlinks),
 	 tab,stats.new_files,stats.new_symlinks);
    if(stats.mod_files || stats.mod_symlinks)
-      printf(plural("%sModified: %d file$|s$, %d symlink$|s$\n",
+      s.appendf(plural("%sModified: %d file$|s$, %d symlink$|s$\n",
 		     stats.mod_files,stats.mod_symlinks),
 	 tab,stats.mod_files,stats.mod_symlinks);
    if(stats.bytes)
-      printf("%s%s\n",tab,CopyJob::FormatBytesTimeRate(stats.bytes,stats.time));
+      s.appendf("%s%s\n",tab,CopyJob::FormatBytesTimeRate(stats.bytes,stats.time));
    if(stats.del_dirs || stats.del_files || stats.del_symlinks)
-      printf(plural(flags&DELETE ?
+      s.appendf(plural(flags&DELETE ?
 	       "%sRemoved: %d director$y|ies$, %d file$|s$, %d symlink$|s$\n"
 	      :"%sTo be removed: %d director$y|ies$, %d file$|s$, %d symlink$|s$\n",
 	      stats.del_dirs,stats.del_files,stats.del_symlinks),
 	 tab,stats.del_dirs,stats.del_files,stats.del_symlinks);
    if(stats.error_count)
-      printf(plural("%s%d error$|s$ detected\n",stats.error_count),
+      s.appendf(plural("%s%d error$|s$ detected\n",stats.error_count),
 	       tab,stats.error_count);
+   return s;
 }
 
 void  MirrorJob::ShowRunStatus(const SMTaskRef<StatusLine>& s)
@@ -348,7 +346,7 @@ void  MirrorJob::HandleFile(FileInfo *file)
 	 }
 	 if(target_is_local && !script_only)
 	 {
-	    if(lstat(target_name,&st)!=-1)
+	    if((flags&RETR_SYMLINKS?stat:lstat)(target_name,&st)!=-1)
 	    {
 	       if(S_ISDIR(st.st_mode))
 	       {
@@ -549,6 +547,13 @@ void  MirrorJob::InitSets(const FileSet *source,const FileSet *dest)
       to_transfer->SubtractAny(to_rm_mismatched);
 
    to_transfer->SortByPatternList(ResMgr::Query("mirror:order",0));
+   to_transfer->CountBytes(&bytes_to_transfer);
+   if(parent_mirror)
+      parent_mirror->AddBytesToTransfer(bytes_to_transfer);
+
+   int dir_count=0;
+   to_transfer->Count(&dir_count,NULL,NULL,NULL);
+   only_dirs = (dir_count==to_transfer->count());
 }
 
 void MirrorJob::HandleChdir(FileAccessRef& session, int &redirections)
@@ -693,7 +698,7 @@ int   MirrorJob::Do()
       if(target_is_local)
       {
 	 struct stat st;
-	 if(lstat(target_dir,&st)!=-1)
+	 if((flags&RETR_SYMLINKS?stat:lstat)(target_dir,&st)!=-1)
 	 {
 	    if(S_ISDIR(st.st_mode))
 	    {
@@ -793,7 +798,8 @@ int   MirrorJob::Do()
       if(FlagSet(DEPTH_FIRST) && source_set && !target_set)
       {
 	 // transfer directories first
-	 to_transfer=new FileSet(source_set);
+	 InitSets(source_set,NULL);
+	 to_transfer->Unsort();
 	 to_transfer->SubtractNotDirs();
 	 goto pre_WAITING_FOR_TRANSFER;
       }
@@ -817,7 +823,9 @@ int   MirrorJob::Do()
    case(WAITING_FOR_TRANSFER):
       while((j=FindDoneAwaitedJob())!=0)
       {
-	 stats.bytes+=j->GetBytesCount();
+	 long long bytes_count=j->GetBytesCount();
+	 AddBytesTransferred(bytes_count);
+	 stats.bytes+=bytes_count;
 	 stats.time +=j->GetTimeSpent();
 	 if(j->ExitCode()!=0)
 	    stats.error_count++;
@@ -843,8 +851,7 @@ int   MirrorJob::Do()
 
 	       // if we have not created any subdirs and there are only subdirs,
 	       // then the directory would be empty - skip it.
-	       if(FlagSet(NO_EMPTY_DIRS) && stats.dirs==0
-	       && source_set->get_fnum()==to_transfer->get_fnum())
+	       if(FlagSet(NO_EMPTY_DIRS) && stats.dirs==0 && only_dirs)
 		  goto pre_FINISHING;
 
 	       transfer_count+=root_transfer_count;
@@ -1099,6 +1106,7 @@ MirrorJob::MirrorJob(MirrorJob *parent,
    FileAccess *source,FileAccess *target,
    const char *new_source_dir,const char *new_target_dir)
  :
+   bytes_transferred(0), bytes_to_transfer(0),
    source_dir(new_source_dir), target_dir(new_target_dir),
    root_transfer_count(0),
    transfer_count(parent?parent->transfer_count:root_transfer_count)
@@ -1410,6 +1418,7 @@ CMD(mirror)
    int	 use_pget=ResMgr::Query("mirror:use-pget-n",0);
    bool	 reverse=false;
    bool	 script_only=false;
+   bool	 no_empty_dirs=ResMgr::QueryBool("mirror:no-empty-dirs",0);
    const char *script_file=0;
    const char *on_change=0;
 
@@ -1589,6 +1598,7 @@ CMD(mirror)
 	 flags|=MirrorJob::ONLY_EXISTING;
 	 break;
       case(OPT_NO_EMPTY_DIRS):
+	 no_empty_dirs=true;
 	 flags|=MirrorJob::NO_EMPTY_DIRS|MirrorJob::DEPTH_FIRST;
 	 break;
       case(OPT_DEPTH_FIRST):
@@ -1654,11 +1664,9 @@ CMD(mirror)
 	 {
 	    // user wants source dir name appended.
 	    const char *base=basename_ptr(source_dir);
-	    if(base[0]!='/' && strcmp(base,basename_ptr(arg)))
-	    {
-	       char *target_dir1=alloca_strdup2(target_dir,strlen(base));
-	       strcat(target_dir1,base);
-	       target_dir=target_dir1;
+	    if(base[0]!='/' && strcmp(base,basename_ptr(arg))) {
+	       target_dir=xstring::cat(target_dir,base,NULL);
+	       target_dir=alloca_strdup(target_dir); // save the buffer
 	    }
 	 }
       }
@@ -1667,8 +1675,10 @@ CMD(mirror)
 	 target_dir=basename_ptr(source_dir);
 	 if(target_dir[0]=='/')
 	    target_dir=".";
-	 else if(target_dir[0]=='~')
-	    target_dir=alloca_strdup(dir_file(".",target_dir));
+	 else if(target_dir[0]=='~') {
+	    target_dir=dir_file(".",target_dir);
+	    target_dir=alloca_strdup(target_dir); // save the buffer
+	 }
       }
    }
 
@@ -1687,6 +1697,9 @@ CMD(mirror)
 	 target_session=parent->session->Clone();
    }
 
+   if(no_empty_dirs)
+      flags|=MirrorJob::NO_EMPTY_DIRS|MirrorJob::DEPTH_FIRST;
+
    Ref<MirrorJob> j(new MirrorJob(0,source_session.borrow(),target_session.borrow(),source_dir,target_dir));
    j->SetFlags(flags,1);
    j->SetVerbose(verbose);
