@@ -1,7 +1,7 @@
 /*
- * lftp and utils
+ * lftp - file transfer program
  *
- * Copyright (c) 1996-2010 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2012 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -14,12 +14,9 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* $Id: MirrorJob.h,v 1.56 2011/03/25 14:35:42 lav Exp $ */
-
 #ifndef MIRRORJOB_H
 #define MIRRORJOB_H
 
@@ -27,6 +24,7 @@
 #include "FileSet.h"
 #include "Job.h"
 #include "PatternSet.h"
+#include "misc.h"
 
 class MirrorJob : public Job
 {
@@ -52,6 +50,8 @@ class MirrorJob : public Job
    bool target_is_local;
    bool source_is_local;
 
+   long long bytes_transferred;
+   long long bytes_to_transfer;
    Ref<FileSet> target_set;
    Ref<FileSet> source_set;
    Ref<FileSet> to_transfer;
@@ -61,6 +61,27 @@ class MirrorJob : public Job
    Ref<FileSet> old_files_set;
    Ref<FileSet> new_files_set;
    void	 InitSets(const FileSet *src,const FileSet *dst);
+   bool only_dirs;
+
+   void AddBytesTransferred(long long b) {
+      bytes_transferred+=b;
+      if(parent_mirror)
+	 parent_mirror->AddBytesTransferred(b);
+   }
+   void AddBytesToTransfer(long long b) {
+      bytes_to_transfer+=b;
+      if(parent_mirror)
+	 parent_mirror->AddBytesToTransfer(b);
+   }
+
+   const xstring& GetCmdLine() const {
+      if(bytes_to_transfer>0) {
+	 return xstring::get_tmp(cmdline).appendf(" - %lld/%lld (%d%%)",
+	    bytes_transferred,bytes_to_transfer,
+	    percent(bytes_transferred,bytes_to_transfer));
+      }
+      return cmdline;
+   }
 
    void	 HandleFile(FileInfo *);
 
@@ -177,9 +198,9 @@ public:
    int	 Do();
    int	 Done() { return state==DONE; }
    void	 ShowRunStatus(const SMTaskRef<StatusLine>&);
-   void	 PrintStatus(int v,const char *);
+   xstring& FormatStatus(xstring&,int v,const char *);
    void	 SayFinal() { PrintStatus(0,""); }
-   int	 ExitCode() { return stats.error_count; }
+   int	 ExitCode() { return stats.error_count!=0; }
 
    void	 SetExclude(PatternSet *x) { my_exclude=x; exclude=my_exclude; }
    void	 SetExclude(const PatternSet *x) { exclude=x; }
