@@ -1,7 +1,7 @@
 /*
  * lftp - file transfer program
  *
- * Copyright (c) 1996-2012 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2013 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,8 +17,6 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* $Id: SFtp.cc,v 1.90 2011/03/25 14:35:43 lav Exp $ */
-
 #include <config.h>
 #include "SFtp.h"
 #include "ArgV.h"
@@ -327,6 +325,9 @@ void SFtp::Disconnect()
    recv_translate=0;
    ssh_id=0;
    home_auto.set(FindHomeAuto());
+   // may have to resend file info queries.
+   if(fileset_for_info)
+      fileset_for_info->rewind();
 }
 
 void SFtp::Init()
@@ -663,13 +664,22 @@ void SFtp::SendRequest()
 
 void SFtp::SendArrayInfoRequests()
 {
-   while(array_ptr<array_cnt && RespQueueSize()<max_packets_in_flight)
+   for(FileInfo *fi=fileset_for_info->curr();
+      fi && RespQueueSize()<max_packets_in_flight;
+      fi=fileset_for_info->next())
    {
-      SendRequest(new Request_STAT(lc_to_utf8(dir_file(cwd,
-	       array_for_info[array_ptr].file)),
-	 SSH_FILEXFER_ATTR_SIZE|SSH_FILEXFER_ATTR_MODIFYTIME,
-	 protocol_version),Expect::INFO,array_ptr);
-      array_ptr++;
+      if(fi->need&(fi->SIZE|fi->DATE|fi->MODE|fi->TYPE|fi->USER|fi->GROUP)) {
+	 unsigned flags=0;
+	 if(fi->need&fi->SIZE) flags|=SSH_FILEXFER_ATTR_SIZE;
+	 if(fi->need&fi->DATE) flags|=SSH_FILEXFER_ATTR_MODIFYTIME;
+	 if(fi->need&fi->MODE) flags|=SSH_FILEXFER_ATTR_PERMISSIONS;
+	 if(fi->need&(fi->USER|fi->GROUP)) flags|=SSH_FILEXFER_ATTR_OWNERGROUP;
+	 SendRequest(new Request_STAT(WirePath(fi->name),flags,
+	    protocol_version),Expect::INFO,fileset_for_info->curr_index());
+      }
+      if(fi->need&fi->SYMLINK_DEF && protocol_version>=3)
+	 SendRequest(new Request_READLINK(WirePath(fi->name)),
+	    Expect::INFO_READLINK,fileset_for_info->curr_index());
    }
    if(RespQueueIsEmpty())
       state=DONE;
@@ -805,8 +815,10 @@ void SFtp::HandleExpect(Expect *e)
       if(reply->TypeIs(SSH_FXP_ATTRS))
       {
 	 const FileAttrs *a=((Reply_ATTRS*)reply)->GetAttrs();
-	 if(a->type!=SSH_FILEXFER_TYPE_DIRECTORY)
+	 if(a->type!=SSH_FILEXFER_TYPE_DIRECTORY
+	 && a->type!=SSH_FILEXFER_TYPE_SYMLINK)	// workaround for RouterOS v6
 	 {
+	    LogError(1,"got file type %d",a->type);
 	    cache->SetDirectory(this,cwd,false);
 	    SetError(NO_FILE,strerror(ENOTDIR));
 	    break;
@@ -949,6 +961,12 @@ void SFtp::HandleExpect(Expect *e)
       }
       break;
    case Expect::INFO:
+      if(mode==ARRAY_INFO)
+      {
+	 FileInfo *fi=(*fileset_for_info)[e->i];
+	 MergeAttrs(fi,((Reply_ATTRS*)reply)->GetAttrs());
+	 break;
+      }
       entity_size=NO_SIZE;
       entity_date=NO_DATE;
       if(reply->TypeIs(SSH_FXP_ATTRS))
@@ -960,19 +978,23 @@ void SFtp::HandleExpect(Expect *e)
 	    entity_date=a->mtime;
 	 LogNote(9,"file info: size=%lld, date=%s",(long long)entity_size,ctime(&entity_date));
       }
-      if(mode==ARRAY_INFO)
-      {
-	 array_for_info[e->i].size=entity_size;
-	 array_for_info[e->i].get_size=false;
-	 array_for_info[e->i].time=entity_date;
-	 array_for_info[e->i].get_time=false;
-	 break;
-      }
       if(opt_size)
 	 *opt_size=entity_size;
       if(opt_date)
 	 *opt_date=entity_date;
       break;
+   case Expect::INFO_READLINK:
+      if(reply->TypeIs(SSH_FXP_NAME)) {
+	 Reply_NAME *r=(Reply_NAME*)reply;
+	 const NameAttrs *a=r->GetNameAttrs(0);
+	 LogNote(9,"file info: symlink=%s",a->name.get());
+	 if(mode==ARRAY_INFO)
+	 {
+	    FileInfo *fi=(*fileset_for_info)[e->i];
+	    fi->SetSymlink(a->name);
+	 }
+      }
+      break;
    case Expect::WRITE_STATUS:
       if(reply->TypeIs(SSH_FXP_STATUS))
       {
@@ -1136,6 +1158,7 @@ void SFtp::CloseExpectQueue()
 	 break;
       case Expect::CWD:
       case Expect::INFO:
+      case Expect::INFO_READLINK:
       case Expect::DEFAULT:
       case Expect::DATA:
       case Expect::WRITE_STATUS:
@@ -1274,6 +1297,7 @@ int SFtp::Done()
 
 void SFtp::SuspendInternal()
 {
+   super::SuspendInternal();
    if(recv_buf)
       recv_buf->SuspendSlave();
    if(send_buf)
@@ -1293,6 +1317,7 @@ void SFtp::ResumeInternal()
       pty_send_buf->ResumeSlave();
    if(pty_recv_buf)
       pty_recv_buf->ResumeSlave();
+   super::ResumeInternal();
 }
 
 const char *SFtp::CurrentStatus()
@@ -1984,6 +2009,36 @@ FileSet *SFtp::GetFileSet()
    return fset?fset:new FileSet;
 }
 
+void SFtp::MergeAttrs(FileInfo *fi,const FileAttrs *a)
+{
+   switch(a->type)
+   {
+   case SSH_FILEXFER_TYPE_REGULAR:  fi->SetType(fi->NORMAL);    break;
+   case SSH_FILEXFER_TYPE_DIRECTORY:fi->SetType(fi->DIRECTORY); break;
+   case SSH_FILEXFER_TYPE_SYMLINK:  fi->SetType(fi->SYMLINK);   break;
+   default: break;
+   }
+   if(a->flags&SSH_FILEXFER_ATTR_SIZE)
+      fi->SetSize(a->size);
+   if(a->flags&SSH_FILEXFER_ATTR_UIDGID)
+   {
+      char id[24];
+      snprintf(id,sizeof(id),"%u",a->uid);
+      fi->SetUser(id);
+      snprintf(id,sizeof(id),"%u",a->gid);
+      fi->SetGroup(id);
+   }
+   if(a->flags&SSH_FILEXFER_ATTR_OWNERGROUP)
+   {
+      fi->SetUser (utf8_to_lc(a->owner));
+      fi->SetGroup(utf8_to_lc(a->group));
+   }
+   if(a->flags&SSH_FILEXFER_ATTR_PERMISSIONS)
+      fi->SetMode(a->permissions&07777);
+   if(a->flags&SSH_FILEXFER_ATTR_MODIFYTIME)
+      fi->SetDate(a->mtime,0);
+}
+
 FileInfo *SFtp::MakeFileInfo(const NameAttrs *na)
 {
    const FileAttrs *a=&na->attrs;
@@ -2001,30 +2056,16 @@ FileInfo *SFtp::MakeFileInfo(const NameAttrs *na)
    Ref<FileInfo> fi(new FileInfo(name));
    switch(a->type)
    {
-   case SSH_FILEXFER_TYPE_REGULAR:  fi->SetType(fi->NORMAL);    break;
-   case SSH_FILEXFER_TYPE_DIRECTORY:fi->SetType(fi->DIRECTORY); break;
-   case SSH_FILEXFER_TYPE_SYMLINK:  fi->SetType(fi->SYMLINK);   break;
+   case SSH_FILEXFER_TYPE_REGULAR:
+   case SSH_FILEXFER_TYPE_DIRECTORY:
+   case SSH_FILEXFER_TYPE_SYMLINK:
    case SSH_FILEXFER_TYPE_UNKNOWN: break;
    default: return 0;
    }
    if(longname)
       fi->SetLongName(longname);
-   if(a->flags&SSH_FILEXFER_ATTR_SIZE)
-      fi->SetSize(a->size);
-   if(a->flags&SSH_FILEXFER_ATTR_UIDGID)
-   {
-      char id[24];
-      snprintf(id,sizeof(id),"%u",a->uid);
-      fi->SetUser(id);
-      snprintf(id,sizeof(id),"%u",a->gid);
-      fi->SetGroup(id);
-   }
-   if(a->flags&SSH_FILEXFER_ATTR_OWNERGROUP)
-   {
-      fi->SetUser (utf8_to_lc(a->owner));
-      fi->SetGroup(utf8_to_lc(a->group));
-   }
-   else if(fi->longname)
+   MergeAttrs(fi.get_non_const(),a);
+   if(fi->longname && !a->owner)
    {
       // try to extract owner/group from long name.
       Ref<FileInfo> ls(FileInfo::parse_ls_line(fi->longname,0));
@@ -2038,10 +2079,6 @@ FileInfo *SFtp::MakeFileInfo(const NameAttrs *na)
 	    fi->SetNlink(ls->nlinks);
       }
    }
-   if(a->flags&SSH_FILEXFER_ATTR_PERMISSIONS)
-      fi->SetMode(a->permissions&07777);
-   if(a->flags&SSH_FILEXFER_ATTR_MODIFYTIME)
-      fi->SetDate(a->mtime,0);
    return fi.borrow();
 }
 
@@ -2176,6 +2213,7 @@ const char *SFtpDirList::Status()
 
 void SFtpDirList::SuspendInternal()
 {
+   super::SuspendInternal();
    if(ubuf)
       ubuf->SuspendSlave();
 }
@@ -2183,6 +2221,7 @@ void SFtpDirList::ResumeInternal()
 {
    if(ubuf)
       ubuf->ResumeSlave();
+   super::ResumeInternal();
 }
 
 
@@ -2193,7 +2232,7 @@ int SFtpListInfo::Do()
    int m=STALL;
    if(done)
       return m;
-   if(!ubuf)
+   if(!ubuf && !result)
    {
       const char *cache_buffer=0;
       int cache_buffer_size=0;
@@ -2220,29 +2259,66 @@ int SFtpListInfo::Do()
 	    ubuf->Save(FileAccess::cache->SizeLimit());
       }
    }
-   const char *b;
-   int len;
-   ubuf->Get(&b,&len);
-   if(b==0) // eof
-   {
+   if(!result) {
+      const char *b;
+      int len;
+      ubuf->Get(&b,&len);
+      if(len>0)
+      {
+	 ubuf->Skip(len);
+	 return MOVED;
+      }
+      if(ubuf->Error())
+      {
+	 SetError(ubuf->ErrorText());
+	 return MOVED;
+      }
+      if(b)
+	 return m;
+      // eof
       if(!result && session->IsOpen())
 	 result=session.Cast<SFtp>()->GetFileSet();
       FileAccess::cache->Add(session,"",FA::LONG_LIST,FA::OK,ubuf,result);
-      result->ExcludeDots();
       result->Exclude(exclude_prefix,exclude);
+      m=MOVED;
+   }
+   if(result && session->OpenMode()!=FA::ARRAY_INFO)
+   {
+      ubuf=0;
+      result->rewind();
+      for(FileInfo *file=result->curr(); file!=0; file=result->next())
+      {
+	 file->need=0;
+	 if(file->defined & file->TYPE)
+	 {
+	    if(file->filetype==file->SYMLINK && follow_symlinks)
+	    {
+	       file->defined &= ~(file->SIZE|file->DATE|file->SYMLINK_DEF|file->MODE|file->TYPE|file->USER|file->GROUP);
+	       file->Need(file->SIZE|file->DATE|file->MODE|file->TYPE|file->USER|file->GROUP);
+	    }
+	    else if(file->filetype==file->SYMLINK)
+	    {
+	       // need the link target
+	       if(!file->Has(file->SYMLINK_DEF))
+		  file->Need(file->SYMLINK_DEF);
+	    }
+	 }
+      }
+      session->GetInfoArray(result.get_non_const());
+      session->Roll();
+      m=MOVED;
+   }
+   if(session->OpenMode()==FA::ARRAY_INFO)
+   {
+      int res=session->Done();
+      if(res==FA::DO_AGAIN)
+	 return m;
+      if(res==FA::IN_PROGRESS)
+	 return m;
+      session->Close();
       done=true;
       m=MOVED;
    }
-   if(len>0)
-   {
-      ubuf->Skip(len);
-      m=MOVED;
-   }
-   if(ubuf->Error())
-   {
-      SetError(ubuf->ErrorText());
-      m=MOVED;
-   }
    return m;
 }
 const char *SFtpListInfo::Status()
