@@ -17,8 +17,6 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* $Id: buffer.h,v 1.59 2009/08/05 06:27:14 lav Exp $ */
-
 #ifndef BUFFER_H
 #define BUFFER_H
 
@@ -79,9 +77,11 @@ public:
    void Skip(int len); // Get(); consume; Skip()
    void UnSkip(int len); // this only works if there were no Put's.
    void Append(const char *buf,int size);
+   void Append(const xstring& s) { Append(s.get(),s.length()); }
    void Put(const char *buf,int size);
    void Put(const char *buf) { Put(buf,strlen(buf)); }
    void Put(const xstring &s) { Put(s.get(),s.length()); }
+   void Put(char c) { Put(&c,1); }
    void Format(const char *f,...) PRINTF_LIKE(2,3);
    void vFormat(const char *f, va_list v);
    void PutEOF() { eof=true; }
@@ -168,7 +168,7 @@ protected:
 
 public:
    DirectedBuffer(dir_t m) : mode(m) {}
-   void SetTranslator(DataTranslator *t) { translator=t; }
+   void SetTranslator(DataTranslator *t);
    void SetTranslation(const char *be_encoding,bool translit=true)
 #ifdef HAVE_ICONV
       ;
@@ -195,6 +195,7 @@ protected:
    virtual int PutEOF_LL() { return 0; }
 
    Time event_time; // used to detect timeouts
+   int max_buf;
 
 public:
    IOBuffer(dir_t m);
@@ -219,8 +220,12 @@ public:
    void Put(const char *,int);
    void Put(const char *buf);
    void Put(const xstring &s) { Put(s.get(),s.length()); }
+   void Put(char c) { Put(&c,1); }
    // anchor to PutEOF_LL
    void PutEOF() { DirectedBuffer::PutEOF(); PutEOF_LL(); }
+
+   void SetMaxBuffered(int m) { max_buf=m; }
+   bool IsFull() { return Size()+(translator?translator->Size():0) >= max_buf; }
 };
 
 class IOBufferStacked : public IOBuffer
@@ -230,6 +235,9 @@ class IOBufferStacked : public IOBuffer
    int Get_LL(int size);
    int Put_LL(const char *buf,int size);
 
+   void SuspendInternal();
+   void ResumeInternal();
+
 public:
    IOBufferStacked(IOBuffer *b) : IOBuffer(b->GetDirection()), down(b) {}
    const Time& EventTime() { return down->EventTime(); }
@@ -269,11 +277,12 @@ class IOBufferFileAccess : public IOBuffer
 
    int Get_LL(int size);
 
+   void SuspendInternal();
+   void ResumeInternal();
+
 public:
    IOBufferFileAccess(const FileAccessRef& i) : IOBuffer(GET), session(i) {}
 
-   void SuspendInternal();
-   void ResumeInternal();
    const char *Status();
 };
 
