@@ -17,8 +17,6 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* $Id: resource.cc,v 1.171 2011/04/22 15:43:11 lav Exp $ */
-
 #include <config.h>
 
 #include <pwd.h>
@@ -215,6 +213,7 @@ static ResType lftp_vars[] = {
    {"ftps:initial-prot",	 "",	  ProtValidate,0},
 #endif
    {"ftp:stat-interval",	 "1",	  ResMgr::TimeIntervalValidate,0},
+   {"ftp:strict-multiline",	 "off",	  ResMgr::BoolValidate,0},
    {"ftp:sync-mode",		 "on",    ResMgr::BoolValidate,0},
    {"ftp:trust-feat",		 "no",	  ResMgr::BoolValidate,0},
    {"ftp:use-abor",		 "yes",   ResMgr::BoolValidate,0},
@@ -355,6 +354,7 @@ static ResType lftp_vars[] = {
    {"ssl:cert-file",		 "",	  ResMgr::FileReadable,0},
    {"ssl:check-hostname",	 "yes",	  ResMgr::BoolValidate,0},
    {"ssl:verify-certificate",	 "yes",	  ResMgr::BoolValidate,0},
+   {"ssl:use-sni",		 "yes",	  ResMgr::BoolValidate,0},
 # if USE_OPENSSL
    {"ssl:ca-path",		 "",	  ResMgr::DirReadable,ResMgr::NoClosure},
    {"ssl:crl-path",		 "",	  ResMgr::DirReadable,ResMgr::NoClosure},
